// Sample Add-In BC function. This function should be applied to a Pa vs time BC function and it will 
// return the initial condition air pressure as the fixed Pa value for all time steps. This will 
// let you get the right Pa on a non horizontal surface. It will be the equivalent of a constant air head.
//
// Copyright (C) 2012 GEO-SLOPE International, Ltd.  All rights reserved.
using System;
public class Pa_Fix_as_Pinitial : Gsi.Function
{

	// just give a dummy message so the UI shows how function works.
	public double There_are_no_input_parameters;  	

	// declare the variable to return to the solver.  Its is not public so won't show up as a UI 
	// item.  An instance of it will be owed by each node the BC function is applied to.  It's
	// value will be retained until the function closes.

	double Pa;    	

	// constructor to initialize data.  This only runs when a function is first initialized.	
	public Pa_Fix_as_Pinitial()  
	{
		Pa = 0.0; // set Pa default to be atmospheric pressure
	}

	// x is the passed in time of the Pa vs Time function.  It is not used in this version as Pa is 
	// returned constant for all time.	
	public double Calculate( double x )   
	{
		int ItNum ;  		
		int nStepNum;

		// this will get live data from the solver.
		try
		{
			ItNum = (int)GetParam(Gsi.DataParamType.eStepIterationCount);  		
            nStepNum = (int)GetParam(Gsi.DataParamType.eStepNum);
		}
		catch
		{
			ItNum = 0;  		
			nStepNum = 0;
		}

		// first time called so set up Pa based on the read in initial condition file Pa for
		// this node.
		
		if(ItNum == 1 && nStepNum == 1)		
		{
			// this will get the air pressure at this node.  A copy of this function 
			// exists for each node it is applied as a BC to.
			Pa = GetParam(Gsi.DataParamType.eAirPressure);  
		}
		
		// return the function Pa value
		return(Pa);
	}
}
