// SIGMA/W Add-In Bounday Condition Model.  
//
// Copyright (C) 2012 GEO-SLOPE International, Ltd.  All rights reserved.
using System;

public class Non_Uniform_Edge_Stress_BC : Gsi.Function
{
	// stress distribution input parameters

	public double Xa, Ya, Xb, Yb, Stress_a, Stress_b;  

	// declare the variable to return to the solver.  
	// Its is not public so won't show up as a UI item.  
	// An instance of it will be owed by each node the BC function is applied to.  
	// It's value will be retained until the function closes.

	double Stress;    	

	// Time is the passed in "x" type of the stress function.  
	// Time is not used in this version but it is here because it
	// is the default function type.
	// We call internal coordinates to pass back the stress vs position

	public double Calculate( double time )   
	{

		double X;   // this will get live data from the solver.
		double Y; 

		// attempt to get information from the solver
		try
		{
			X = GetParam(Gsi.DataParamType.eXCoord); 
			Y = GetParam(Gsi.DataParamType.eYCoord);		
		}
		catch 
		{
			// if the solver is not running, use the passed in default "x" value
			// as our X value and the Y value as the users second y coordinate.
			X = time;
			Y = Yb;
		}
		
		double Slope = (Stress_b - Stress_a) / Math.Sqrt( (Yb - Ya)*(Yb - Ya)  + (Xb - Xa)*(Xb - Xa) );
		Stress = Stress_a + Slope * Math.Sqrt( (X - Xa)*(X - Xa)  + ( Y - Ya)*(Y - Ya) );
	
		// return the function Stress value
		return(Stress);
	}
}
