::---------------------------------------------------------------------------------------
:: Command line run C# compiler to create a GeoStudio Add-In Assembly. The Assembly is 
:: placed in the GeoStudio custom Add-In directory
::
:: Command line argument is C# add in file without the file extension. If the file name
:: contains spaces you must put quotes around the name.
::
:: For example to compile foo.cs into an Add-In, go 
:: >gsaddin "foo"
::
:: If you want to create a optimized release build, type in "release" as the second argument. 
:: The default is the create a debug build so you can attach a debugger.
::---------------------------------------------------------------------------------------
@echo off
:: Don't pollute the global enviornment variable namespace
setlocal
set _geostudio_name=GeoStudio 8

set _cmdmode=
set _addin=%~1
if defined _addin (set _cmdmode=1)

:: -------- Find the C# compiler
:: The base allowed version of the .NET CLR used by GeoStudio is .NET 2.0. We make this
:: the default so the AddIn can run on the most possible target systems. If .NET 2.0 is
:: not found, try .NET 4.0. 
set _csc=%windir%\Microsoft.NET\Framework\v2.0.50727\csc.exe
%_csc% /? > NUL
set _exitcode=%ERRORLEVEL%
if %_exitcode% equ 0 (goto CSharpCompilerFound)
 
set _csc=%windir%\Microsoft.NET\Framework\v4.0.30319\csc.exe
%_csc% /? > NUL
set _exitcode=%ERRORLEVEL%
if %_exitcode% equ 0 (goto CSharpCompilerFound)

echo Error: Can't find a supported version of the C# compiler. 
echo        Make sure .NET 2.0 or 4.0 is installed.
goto ExitScript

:CSharpCompilerFound

:: ---- Find the program files path
:: Its assumed GeoStudio is installed here. 64 bit windows uses a different path, so we try 
:: to detect and use that path first.
set _progfilespath=
if exist "%ProgramFiles(x86)%\GEO-SLOPE\*.*" (set _progfilespath="%ProgramFiles(x86)%")
if not defined _progfilespath (set _progfilespath="%ProgramFiles%")
:: Remove quotes: the '(' and ')' characters confuse the cmd script parser unless in quotes above.
:: We use a for, as it parses a quoted string (which _progfilespath is), to stip the quotes.
for /f "tokens=*" %%D in ( %_progfilespath% ) do (set _progfilespath=%%D)

:: ---- Find the GeoStudio installation path
set _gsinstallpath=%_progfilespath%\GEO-SLOPE\%_geostudio_name%
if exist "%_gsinstallpath%" (goto GeoStudioInstallFound)
set _gsinstallpath=%_progfilespath%\GEO-SLOPE\%_geostudio_name% beta
if exist "%_gsinstallpath%" (goto GeoStudioInstallFound)
echo Error: Can't find the GeoStudio install path. 
echo        Make sure GeoStudio 2007 or GeoStudio 2012 is installed in its default location.
goto ExitScript

:GeoStudioInstallFound
:: ---- Find the Add-In destination path
:: We look in the registry to extract the directory name from there. If not found, assume the default 
set _gsaddinpath=%userprofile%\Application Data\GEO-SLOPE\%_geostudio_name%\AddIns
:: In the for that follows, we get a separate token for each space delimited part of the path.
:: Concatentate them all together -- 9 parts should be plenty. Use 2>&1 to force stderror to go 
:: to the same place as stdout ... the NUL device. We use the NUL device from the temp directory so
:: only 1 file handle is ever used.
set _regKeyFound=false
reg query "HKCU\Software\GEO-SLOPE\%_geostudio_name%" /v AddInsDir >%temp%\NUL 2>&1 && set _regKeyFound=true
if %_regKeyFound%==true goto DoRegQuery
goto GetAddIn
: DoRegQuery
for /f "skip=1 tokens=2*" %%P in ('reg query "HKCU\Software\GEO-SLOPE\%_geostudio_name%" /v AddInsDir') do set _gsaddinpath=%%Q
:: remove trailing backslash
if "[%_gsaddinpath:~-1%]"=="[\]" set _gsaddinpath=%_gsaddinpath:~0,-1%
if not exist "%_gsaddinpath%" (
	echo Error: Add-In folder "%_gsaddinpath%" does not exist. 
	echo        Run GeoStudio and choose Edit Options to set the Add-Ins folder.
	goto ExitScript
)
echo Add-In will be saved to "%_gsaddinpath%"

:: ---- Get the Add-In name (source file).
:: Assign AddIn from command line or prompt if not given. If not given, prompt user for an addIn name.
:GetAddIn
if not defined _addin (set /p _addin=AddIn Name:)
:: Just in case no .cs extension is given, provide it.
if not exist "%_addin%" (set _addin=%_addin%.cs)
if not exist "%_addin%" (
	echo Error: File "%_addin%" not found.
	goto :ExitScript
)
:: Get the full file path to the C# source file.
set _addin_fpath=%_addin%
for /f "tokens=*" %%I in ("%_addin%") do set _addin_fpath=%%~fI
:: Extract the file name from the addin string. 
set _addin_fname=%_addin%
for /f "tokens=*" %%I in ("%_addin%") do set _addin_fname=%%~nI

:: -- Determine if building debug or release
:: The debug flag works by doing a debug build if no other argument is given. If a 
:: second argument is provided, it is used.
set _debug=
set _debug_flag=%2
if not defined _debug_flag (set _debug=/debug:full)

:: Include the installed AddIns path only if it exists (it isn't installed with GeoStudio 2012)
if exist "%_gsinstallpath%\AddIns" (
	set _installedLib="/lib:%_gsinstallpath%\AddIns"
)

:: --- Compile AddIn
echo on
%_csc% /target:library %_installedLib% "/lib:%_gsaddinpath%" "/reference:%_gsinstallpath%\bin\mngsrv.dll" "/out:%_gsaddinpath%\%_addin_fname%.dll" %_debug% "%_addin_fpath%"
@echo off
goto ExitScript

:ExitScript
set _answer=
if not defined _cmdmode (set /p _answer=Press Return to Continue)
echo.

